@@ -1,20 +1,10 @@
 /***
+
   This file is part of cups-filters.
 
-  This file is free software; you can redistribute it and/or modify it
-  under the terms of the GNU Lesser General Public License as
-  published by the Free Software Foundation; either version 2.1 of the
-  License, or (at your option) any later version.
+  Licensed under Apache License v2.0.  See the file "LICENSE" for more
+  information.
 
-  This file is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
-  Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with cups-filters; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
-  USA.
 ***/
 
 #ifdef HAVE_CONFIG_H
@@ -30,12 +20,13 @@
 #include <sys/stat.h>
 #include <stdlib.h>
 #include <signal.h>
+#include <fcntl.h>
 #include <sys/wait.h>
 #include <cups/cups.h>
-#include <cups/ppd.h>
+#include <ppd/ppd.h>
 #include <cups/raster.h>
 #include <cupsfilters/ipp.h>
-#include <cupsfilters/ppdgenerator.h>
+#include <cupsfilters/ipp.h>
 
 #define MAX_OUTPUT_LEN 8192
 
@@ -284,17 +275,17 @@ listPrintersInArray(int reg_type_no, int mode, int isFax,
 	   strcasestr(pdl, "image/"))) {
 	value[0] = '\0';
 	if (strcasestr(pdl, "application/pdf"))
-	  strncat(value, ",PDF", sizeof(value));
+	  strncat(value, ",PDF", sizeof(value) - 1);
 	if (strcasestr(pdl, "application/PCLm"))
-	  strncat(value, ",PCLM", sizeof(value));
+	  strncat(value, ",PCLM", sizeof(value) - 1);
 	if (strcasestr(pdl, "application/postscript"))
-	  strncat(value, ",PS", sizeof(value));
+	  strncat(value, ",PS", sizeof(value) - 1);
 	if (strcasestr(pdl, "application/vnd.hp-PCL"))
-	  strncat(value, ",PCL", sizeof(value));
+	  strncat(value, ",PCL", sizeof(value) - 1);
 	if (strcasestr(pdl, "image/pwg-raster"))
-	  strncat(value, ",PWGRaster", sizeof(value));
+	  strncat(value, ",PWGRaster", sizeof(value) - 1);
 	if (strcasestr(pdl, "image/urf"))
-	  strncat(value, ",AppleRaster", sizeof(value));
+	  strncat(value, ",AppleRaster", sizeof(value) - 1);
 	for (ptr = strcasestr(pdl, "image/"); ptr;
 	     ptr = strcasestr(ptr, "image/")) {
 	  char *valptr = value + strlen(value);
@@ -417,16 +408,12 @@ list_printers (int mode, int reg_type_no, int isFax)
   ippfind_argv[i++] = "(";
   ippfind_argv[i++] = "--txt-pdl";        /* PDL list in TXT record contains */
   ippfind_argv[i++] = "image/pwg-raster"; /* PWG Raster (IPP Everywhere) */
-#ifdef QPDF_HAVE_PCLM
   ippfind_argv[i++] = "--or";             /* or */
   ippfind_argv[i++] = "--txt-pdl";
   ippfind_argv[i++] = "application/PCLm"; /* PCLm */
-#endif
-#ifdef CUPS_RASTER_HAVE_APPLERASTER
   ippfind_argv[i++] = "--or";             /* or */
   ippfind_argv[i++] = "--txt-pdl";
   ippfind_argv[i++] = "image/urf";        /* Apple Raster */
-#endif
   ippfind_argv[i++] = "--or";             /* or */
   ippfind_argv[i++] = "--txt-pdl";
   ippfind_argv[i++] = "application/pdf";  /* PDF */
@@ -508,19 +495,20 @@ list_printers (int mode, int reg_type_no, int isFax)
   if (fp) {
     while ((bytes = cupsFileGetLine(fp, buffer, sizeof(buffer))) > 0 ||
 	   (bytes < 0 && (errno == EAGAIN || errno == EINTR))) {
-      ippfind_output = (char *)malloc(MAX_OUTPUT_LEN*(sizeof(char)));
       ptr = buffer;
       while (ptr && !isalnum(*ptr & 255)) ptr ++;
       if ((!strncasecmp(ptr, "ipps", 4) && ptr[4] == '\t')) {
 	ptr += 4;
 	*ptr = '\0';
 	ptr ++;
+	ippfind_output = (char *)malloc(MAX_OUTPUT_LEN*(sizeof(char)));
 	snprintf(ippfind_output, MAX_OUTPUT_LEN, "%s", ptr);
 	cupsArrayAdd(service_uri_list_ipps, ippfind_output);
       } else if ((!strncasecmp(ptr, "ipp", 3) && ptr[3] == '\t')) {
 	ptr += 3;
 	*ptr = '\0';
 	ptr ++;
+	ippfind_output = (char *)malloc(MAX_OUTPUT_LEN*(sizeof(char)));
 	snprintf(ippfind_output, MAX_OUTPUT_LEN, "%s", ptr);
 	cupsArrayAdd(service_uri_list_ipp, ippfind_output);
       } else
@@ -604,7 +592,7 @@ int
 generate_ppd (const char *uri, int isFax)
 {
   ipp_t *response = NULL;
-  char buffer[65536], ppdname[1024];
+  char buffer[65536], ppdname[1024], ppdgenerator_msg[1024];
   int  fd,
        bytes;
   char *ptr1,
@@ -624,10 +612,10 @@ generate_ppd (const char *uri, int isFax)
   /* Request printer properties via IPP to generate a PPD file for the
      printer */
 
-  response = get_printer_attributes4(uri, NULL, 0, NULL, 0, 1, isFax);
+  response = cfGetPrinterAttributes4(uri, NULL, 0, NULL, 0, 1, isFax);
 
   if (debug) {
-    ptr1 = get_printer_attributes_log;
+    ptr1 = cf_get_printer_attributes_log;
     while(ptr1) {
       ptr2 = strchr(ptr1, '\n');
       if (ptr2) *ptr2 = '\0';
@@ -639,13 +627,13 @@ generate_ppd (const char *uri, int isFax)
   if (response == NULL) {
     fprintf(stderr, "ERROR: Unable to create PPD file: Could not poll "
 	    "sufficient capability info from the printer (%s, %s) via IPP!\n",
-	    uri, resolve_uri(uri));
+	    uri, cfResolveURI(uri));
     goto fail;
   }
 
   /* Generate the PPD file */
-  if (!ppdCreateFromIPP(ppdname, sizeof(ppdname), response, NULL, NULL, 0,
-			0)) {
+  if (!ppdCreatePPDFromIPP(ppdname, sizeof(ppdname), response, NULL, NULL, 0,
+			   0, ppdgenerator_msg, sizeof(ppdgenerator_msg))) {
     if (strlen(ppdgenerator_msg) > 0)
       fprintf(stderr, "ERROR: Unable to create PPD file: %s\n",
 	      ppdgenerator_msg);
@@ -691,6 +679,13 @@ main(int argc, char*argv[]) {
   struct sigaction action;		/* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
+ /*
+  * Do not run at all if the NO_DRIVERLESS_PPDS environment variable is set
+  */
+
+  if (getenv("NO_DRIVERLESS_PPDS"))
+    return (0);
+
  /*
   * Make sure status messages are not buffered...
   */
